# lab4.py

import sys
sys.path.append('..')
from common.core import *
from common.audio import *
from common.synth import *
from common.gfxutil import *

from common.clock import *
from common.metro import *

# This class, MetronomeHack, is similar to Metronome (simplified in that it has no stop function). 
# In this lab, you will hack the Metronome class to make it do more interesting things

# Part 1:
# Instead steady clicks, hack MetronomeHack to play a C major scale.
#
# Part 2: Now, instead of playing a C major scale with straight quarter notes,
# hack the class to play the c-major scale with a "swing 8th" rhythm - that is
# instead of just using a beat length of 240 (eight notes), alternate between
# two beat lengths B1 and B2 such that B1+B2 = 480 and B1 > B2
#
# Part 3: Finally, add a bit of graphics. Each time a note plays, create some 
# graphical output that somehow animates and then disappears or fades away
#
class MetronomeHack(object):
    def __init__(self, sched, synth, channel = 0, patch=(128, 0), pitch = 60):
        super(MetronomeHack, self).__init__()
        self.sched = sched
        self.synth = synth
        self.channel = channel
        self.patch = patch
        self.pitch = pitch
        self.beat_len = kTicksPerQuarter

    def start(self):
        # set up the correct sound (program change)
        self.synth.program(self.channel, self.patch[0], self.patch[1])

        # find the tick of the next beat, and make it "beat aligned"
        now = self.sched.get_tick()
        next_beat = quantize_tick_up(now, self.beat_len)

        # now, post the _noteon function (and remember this command)
        self.sched.post_at_tick(next_beat, self._noteon)

    def _noteon(self, tick, ignore):
        # play the note right now:
        self.synth.noteon(self.channel, self.pitch, 100)

        # post the note off for half a beat later:
        self.sched.post_at_tick(tick + self.beat_len/2, self._noteoff, self.pitch)

        # schedule the next noteon for one beat later
        next_beat = tick + self.beat_len
        self.sched.post_at_tick(next_beat, self._noteon)

    def _noteoff(self, tick, pitch):
        # just turn off the currently sounding note.
        self.synth.noteoff(self.channel, pitch)




class MainWidget(BaseWidget) :
    def __init__(self):
        super(MainWidget, self).__init__()

        self.audio = Audio(2)
        self.synth = Synth('../data/FluidR3_GM.sf2')

        # create TempoMap, AudioScheduler
        self.tempo_map  = SimpleTempoMap(120)
        self.sched = AudioScheduler(self.tempo_map)

        # connect scheduler into audio system
        self.audio.set_generator(self.sched)
        self.sched.set_generator(self.synth)

        # the original metronome:
        self.metro = Metronome(self.sched, self.synth, channel = 0)

        # the hacked metronome:
        self.metro_hack = MetronomeHack(self.sched, self.synth, channel = 1)

        # and text to display our status
        self.label = topleft_label()
        self.add_widget(self.label)

    def on_key_down(self, keycode, modifiers):
        if keycode[1] == 'm':
            self.metro.toggle()

        if keycode[1] == 'h':
            self.metro_hack.start()

    def on_update(self) :
        self.audio.on_update()

        self.label.text = self.sched.now_str() + '\n'
        self.label.text += 'tempo:%d\n' % self.tempo_map.get_tempo()
        self.label.text += 'm: toggle Metronome\n'
        self.label.text += 'h: start MetronomeHack\n'


run(MainWidget)

